import json
import os
import shutil
import subprocess
from urllib.parse import urlparse

from .common import REPO_ROOT

CONFIG_DIR: str = os.path.expanduser("~/.config/skget")
SETTINGS_PATH: str = os.path.join(CONFIG_DIR, "settings.json")
DEFAULT_SETTINGS_PATH: str = os.path.join(REPO_ROOT, "default.settings.json")


def _run_git(args: list[str], cwd: str | None = None) -> None:
    try:
        subprocess.run(
            ["git", *args],
            cwd=cwd,
            check=True,
            capture_output=True,
            text=True,
        )
    except FileNotFoundError as exc:
        raise RuntimeError("git is required to clone skills from URL") from exc
    except subprocess.CalledProcessError as exc:
        detail = exc.stderr.strip() or exc.stdout.strip()
        msg = "git command failed"
        if detail:
            msg = f"{msg}: {detail}"
        raise RuntimeError(msg) from exc


def _parse_github_tree_url(url: str) -> tuple[str, str, str, str, str | None, str]:
    parsed = urlparse(url)
    parts = [part for part in parsed.path.split("/") if part]
    if len(parts) < 2:
        raise RuntimeError(f"invalid github skills url: {url}")
    owner = parts[0]
    repo = parts[1]
    repo_name = repo[:-4] if repo.endswith(".git") else repo
    repo_url = f"{parsed.scheme}://{parsed.netloc}/{owner}/{repo_name}"
    branch = None
    subpath = ""
    if len(parts) >= 4 and parts[2] == "tree":
        branch = parts[3]
        if len(parts) > 4:
            subpath = "/".join(parts[4:])
    return repo_url, parsed.netloc, owner, repo_name, branch, subpath


def _ensure_repo(
    repo_url: str,
    clone_root: str,
    branch: str | None,
    subpaths: list[str],
) -> None:
    if not os.path.isdir(clone_root):
        os.makedirs(os.path.dirname(clone_root), exist_ok=True)
        _run_git(["clone", "--filter=blob:none", "--no-checkout", repo_url, clone_root])
    elif not os.path.isdir(os.path.join(clone_root, ".git")):
        raise RuntimeError(f"existing path is not a git repo: {clone_root}")

    if subpaths:
        _run_git(["-C", clone_root, "sparse-checkout", "init", "--cone"])
        _run_git(["-C", clone_root, "sparse-checkout", "set", *subpaths])
    else:
        sparse_file = os.path.join(clone_root, ".git", "info", "sparse-checkout")
        if os.path.isfile(sparse_file):
            _run_git(["-C", clone_root, "sparse-checkout", "disable"])

    if branch:
        try:
            _run_git(["-C", clone_root, "checkout", branch])
        except RuntimeError:
            _run_git(["-C", clone_root, "fetch", "origin", branch])
            _run_git(["-C", clone_root, "checkout", branch])
    elif not os.path.isdir(os.path.join(clone_root, ".git", "refs")):
        _run_git(["-C", clone_root, "checkout"])


def resolve_skills_root(entry: str) -> str:
    parsed = urlparse(entry)
    if parsed.scheme in ("http", "https") and parsed.netloc:
        if parsed.netloc.lower() == "github.com":
            repo_url, host, owner, repo, branch, subpath = _parse_github_tree_url(entry)
            clone_root = (
                os.path.join(CONFIG_DIR, host, owner, repo, "tree", branch)
                if branch
                else os.path.join(CONFIG_DIR, host, owner, repo)
            )
            _ensure_repo(repo_url, clone_root, branch, [subpath] if subpath else [])
            return os.path.join(clone_root, subpath) if subpath else clone_root

        parts = [part for part in parsed.path.split("/") if part]
        if not parts:
            raise RuntimeError(f"invalid skills url: {entry}")
        repo = parts[-1]
        repo_name = repo[:-4] if repo.endswith(".git") else repo
        clone_root = os.path.join(CONFIG_DIR, parsed.netloc, *parts[:-1], repo_name)
        repo_url = (
            f"{parsed.scheme}://{parsed.netloc}/{'/'.join(parts[:-1] + [repo_name])}"
        )
        _ensure_repo(repo_url, clone_root, None, [])
        return clone_root

    expanded = os.path.expanduser(entry)
    return os.path.abspath(expanded)


def _load_settings() -> dict:
    if not os.path.isfile(SETTINGS_PATH):
        if not os.path.isfile(DEFAULT_SETTINGS_PATH):
            return {}
        os.makedirs(CONFIG_DIR, exist_ok=True)
        try:
            shutil.copyfile(DEFAULT_SETTINGS_PATH, SETTINGS_PATH)
        except OSError as exc:
            raise RuntimeError(
                f"unable to write settings file: {SETTINGS_PATH}"
            ) from exc
    try:
        with open(SETTINGS_PATH, "r", encoding="utf-8") as handle:
            data = json.load(handle)
    except (OSError, json.JSONDecodeError) as exc:
        raise RuntimeError(f"invalid settings file: {SETTINGS_PATH}") from exc
    if not isinstance(data, dict):
        raise RuntimeError(f"invalid settings file: {SETTINGS_PATH}")
    return data


def get_skills_roots() -> list[str]:
    settings = _load_settings()
    paths = settings.get("paths")
    if isinstance(paths, dict) and "skills" in paths:
        skills_value = paths.get("skills")
        if not isinstance(skills_value, list):
            raise RuntimeError("invalid settings: paths.skills must be a list")
        roots: list[str] = []
        github_entries: dict[tuple[str, str, str | None], dict[str, object]] = {}

        for entry in skills_value:
            if not isinstance(entry, str):
                continue
            parsed = urlparse(entry)
            if parsed.scheme in ("http", "https") and parsed.netloc.lower() == "github.com":
                repo_url, host, owner, repo, branch, subpath = _parse_github_tree_url(
                    entry
                )
                clone_root = (
                    os.path.join(CONFIG_DIR, host, owner, repo, "tree", branch)
                    if branch
                    else os.path.join(CONFIG_DIR, host, owner, repo)
                )
                key = (repo_url, clone_root, branch)
                if key not in github_entries:
                    github_entries[key] = {"full": False, "subpaths": []}
                if subpath:
                    subpaths = github_entries[key]["subpaths"]
                    if subpath not in subpaths:
                        subpaths.append(subpath)
                else:
                    github_entries[key]["full"] = True
                continue

            roots.append(resolve_skills_root(entry))

        for (repo_url, clone_root, branch), info in github_entries.items():
            full_checkout = bool(info["full"])
            subpaths = info["subpaths"]
            if full_checkout:
                _ensure_repo(repo_url, clone_root, branch, [])
                roots.append(clone_root)
            else:
                _ensure_repo(repo_url, clone_root, branch, subpaths)
                for subpath in subpaths:
                    roots.append(os.path.join(clone_root, subpath))

        return roots

    fallback = os.path.join(REPO_ROOT, "skills")
    if os.path.isdir(fallback):
        return [fallback]
    return []



import os
import re

from tabulate import tabulate

from .common import (
    CLAUDE_BASE,
    CODEX_BASE,
)
from .settings import get_skills_roots, resolve_skills_root

import click

CLAUDE_SKILLS_ROOT: str = os.path.join(CLAUDE_BASE, "skills")
CODEX_SKILLS_ROOT: str = os.path.join(CODEX_BASE, "skills")


def _available_skills() -> list[tuple[str, str]]:
    skills: list[tuple[str, str]] = []
    seen_paths: set[str] = set()
    for skills_root in get_skills_roots():
        if not os.path.isdir(skills_root):
            continue
        for entry in os.listdir(skills_root):
            if entry.startswith("."):
                continue
            entry_path = os.path.join(skills_root, entry)
            if entry_path in seen_paths:
                continue
            if os.path.isdir(entry_path) and os.path.isfile(
                os.path.join(entry_path, "SKILL.md")
            ):
                skills.append((entry, entry_path))
                seen_paths.add(entry_path)
    return sorted(skills, key=lambda item: (item[0], item[1]))


def _installed_destinations(skill_name: str, skill_path: str) -> list[str]:
    destinations: list[str] = []
    for label, root in (
        ("claude", CLAUDE_SKILLS_ROOT),
        ("codex", CODEX_SKILLS_ROOT),
    ):
        target_path = os.path.join(root, skill_name)
        if not os.path.islink(target_path):
            continue
        resolved = os.path.realpath(target_path)
        if os.path.realpath(skill_path) == resolved:
            destinations.append(label)
    return destinations


def _resolve_skill_path(skill_name: str, source_root: str | None = None) -> str:
    if source_root:
        resolved_root = resolve_skills_root(source_root)
        if not os.path.isdir(resolved_root):
            raise RuntimeError(f"invalid source root: {source_root}")
        candidate = os.path.join(resolved_root, skill_name)
        if os.path.isdir(candidate) and os.path.isfile(
            os.path.join(candidate, "SKILL.md")
        ):
            return candidate
        raise RuntimeError(
            f"skill not found in source root: {skill_name} ({resolved_root})"
        )

    for skills_root in get_skills_roots():
        candidate = os.path.join(skills_root, skill_name)
        if os.path.isdir(candidate) and os.path.isfile(
            os.path.join(candidate, "SKILL.md")
        ):
            return candidate
    raise RuntimeError(f"skill not found: {skill_name}")


def _resolve_targets(destination: str) -> list[str]:
    dest = destination.lower()
    if dest == "auto":
        targets: list[str] = []
        if os.path.isdir(CLAUDE_BASE):
            targets.append(CLAUDE_SKILLS_ROOT)
        if os.path.isdir(CODEX_BASE):
            targets.append(CODEX_SKILLS_ROOT)
        if not targets:
            raise RuntimeError("no ~/.claude or ~/.codex directories found")
        return targets
    elif dest == "claude":
        if not os.path.isdir(CLAUDE_BASE):
            raise RuntimeError("~/.claude directory not found")
        return [CLAUDE_SKILLS_ROOT]
    elif dest == "codex":
        if not os.path.isdir(CODEX_BASE):
            raise RuntimeError("~/.codex directory not found")
        return [CODEX_SKILLS_ROOT]
    else:
        raise ValueError(f"invalid destination: {destination}")


def _install_to_root(source_path: str, skill_name: str, target_root: str) -> str:
    os.makedirs(target_root, exist_ok=True)
    target_path = os.path.join(target_root, skill_name)

    if os.path.lexists(target_path):
        if os.path.islink(target_path):
            os.unlink(target_path)
        else:
            raise RuntimeError(f"cannot overwrite existing non-symlink: {target_path}")

    os.symlink(source_path, target_path)
    return target_path


def _uninstall_from_root(skill_name: str, target_root: str) -> str | None:
    target_path = os.path.join(target_root, skill_name)
    if not os.path.lexists(target_path):
        return None
    if not os.path.islink(target_path):
        raise RuntimeError(f"cannot uninstall non-symlink: {target_path}")
    os.unlink(target_path)
    return target_path


def skills_install(
    skill_name: str, destination: str, source_root: str | None = None
) -> list[str]:
    source_path = _resolve_skill_path(skill_name, source_root=source_root)

    target_paths: list[str] = []
    for target_root in _resolve_targets(destination):
        target_paths.append(_install_to_root(source_path, skill_name, target_root))
    return target_paths


def skills_uninstall(
    skill_name: str, destination: str, source_root: str | None = None
) -> list[str]:
    if source_root:
        _resolve_skill_path(skill_name, source_root=source_root)
    target_paths: list[str] = []
    for target_root in _resolve_targets(destination):
        removed = _uninstall_from_root(skill_name, target_root)
        if removed is not None:
            target_paths.append(removed)
    if not target_paths:
        raise RuntimeError(f"skill not installed: {skill_name}")
    return target_paths


def skills_list(
    pattern: str | None = None, installed_only: bool = False
) -> list[tuple[str, str, list[str]]]:
    skills = _available_skills()
    if pattern:
        try:
            matcher = re.compile(pattern)
        except re.error as exc:
            raise ValueError(f"invalid regex: {pattern}") from exc
        skills = [
            (skill_name, skill_path)
            for skill_name, skill_path in skills
            if matcher.search(skill_name)
        ]

    results: list[tuple[str, str, list[str]]] = []
    for skill_name, skill_path in skills:
        destinations = _installed_destinations(skill_name, skill_path)
        if installed_only and not destinations:
            continue
        results.append((skill_name, skill_path, destinations))
    return results


@click.group()
def skills() -> None:
    pass


@skills.command("install")
@click.argument("skill_name")
@click.option(
    "--source-root",
    "source_root",
    default=None,
    help="Override the skills root to install from (path or URL).",
)
@click.option(
    "--dest",
    "-d",
    "destination",
    type=click.Choice(["auto", "claude", "codex"], case_sensitive=False),
    default="auto",
    show_default=True,
)
def skills_install_cmd(skill_name: str, destination: str, source_root: str | None) -> None:
    try:
        for target_path in skills_install(
            skill_name, destination, source_root=source_root
        ):
            click.echo(f"installed {skill_name} -> {target_path}")
    except (RuntimeError, ValueError) as exc:
        raise click.ClickException(str(exc)) from exc


@skills.command("uninstall")
@click.argument("skill_name")
@click.option(
    "--source-root",
    "source_root",
    default=None,
    help="Override the skills root to uninstall from (path or URL).",
)
@click.option(
    "--dest",
    "-d",
    "destination",
    type=click.Choice(["auto", "claude", "codex"], case_sensitive=False),
    default="auto",
    show_default=True,
)
def skills_uninstall_cmd(
    skill_name: str, destination: str, source_root: str | None
) -> None:
    try:
        for target_path in skills_uninstall(
            skill_name, destination, source_root=source_root
        ):
            click.echo(f"uninstalled {skill_name} -> {target_path}")
    except (RuntimeError, ValueError) as exc:
        raise click.ClickException(str(exc)) from exc


@skills.command("list")
@click.argument("pattern", required=False)
@click.option(
    "--installed",
    is_flag=True,
    default=False,
    help="Only list skills installed in any destination.",
)
def skills_list_cmd(pattern: str | None, installed: bool) -> None:
    try:
        # TODO: show skill description
        rows = []
        for skill_name, skill_path, destinations in skills_list(
            pattern=pattern, installed_only=installed
        ):
            dest_label = ",".join(destinations) if destinations else "-"
            rows.append((skill_name, dest_label, skill_path))

        table = tabulate(
            rows,
            headers=["[skill name]", "[installed]", "[source path]"],
            tablefmt="plain",
        )
        if table:
            click.echo(table)
    except (RuntimeError, ValueError) as exc:
        raise click.ClickException(str(exc)) from exc



from __future__ import annotations

import click
import os
import re
import frontmatter
from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.containers import Vertical
from textual.widgets import (
    DataTable,
    Footer,
    Input,
    Static,
    Tab,
    Tabs,
)

from skget.common import CLAUDE_BASE, CODEX_BASE
from skget.skills import skills, skills_install, skills_list, skills_uninstall

LOGO = r"""
███████╗██╗  ██╗ ██████╗ ███████╗████████╗
██╔════╝██║ ██╔╝██╔════╝ ██╔════╝╚══██╔══╝
███████╗█████╔╝ ██║  ███╗█████╗     ██║   
╚════██║██╔═██╗ ██║   ██║██╔══╝     ██║   
███████║██║  ██╗╚██████╔╝███████╗   ██║   
╚══════╝╚═╝  ╚═╝ ╚═════╝ ╚══════╝   ╚═╝   
"""


class SkillsTable(DataTable):
    BINDINGS = DataTable.BINDINGS + [
        Binding("j", "cursor_down", "Down", show=True),
        Binding("k", "cursor_up", "Up", show=True),
        Binding("d", "page_down", "Page down", show=True),
        Binding("u", "page_up", "Page up", show=True),
    ]


class SkgetApp(App):
    ENABLE_COMMAND_PALETTE = False

    CSS = """
    Screen {
        background: $surface;
    }

    #header {
        text-style: bold;
        text-align: center;
        color: $primary;
        background: $boost;
        height: auto;
    }

    DataTable {
        height: 1fr;
        padding: 1 2;
    }

    #search-bar {
        height: 3;
        padding: 0 2;
        display: none;
    }

    #skills-tabs {
        height: 3;
        padding: 0 2;
        display: none;
    }

    #skill-info {
        height: auto;
        padding: 1 2;
        color: $text;
        background: $panel;
        display: none;
    }
    """

    BINDINGS = [
        Binding("enter", "select_row", "Select", show=False, priority=True),
        Binding("space", "select_row_alt", "Select", show=True, priority=True),
        Binding("escape", "go_back", "Back", show=False),
        Binding("backspace", "go_back_alt", "Back", show=True),
        Binding("/", "focus_search", "Search", show=True),
        Binding("tab", "cycle_tab", "Tab", show=True, priority=True),
        Binding("shift+tab", "cycle_tab_reverse", "Tab Back", show=False, priority=True),
        ("q", "quit", "Quit"),
        ("ctrl+q", "ignore", ""),
    ]

    def compose(self) -> ComposeResult:
        with Vertical():
            yield Static(
                LOGO,
                id="header",
            )
            yield Tabs(
                Tab("Skills", id="skills"),
                Tab("Installed", id="installed"),
                id="skills-tabs",
            )
            yield Input(placeholder="Press / to search", id="search-bar")
            yield Static("", id="skill-info")
            yield SkillsTable(id="main-table")
            yield Footer()

    def on_mount(self) -> None:
        self.view_stack: list[str] = []
        self.current_skill: str | None = None
        self.current_skill_path: str | None = None
        self.current_skill_destinations: list[str] | None = None
        self.skill_destinations: dict[str, list[str]] = {}
        self.pending_action: tuple[str, str] | None = None
        self.all_skills_data: list[tuple[str, str, list[str]]] = []
        self.active_tab: str = "skills"
        self._show_skills_view()

    def _show_skills_view(self, refresh: bool = False) -> None:
        if not refresh:
            self.view_stack.append("skills")
        self.current_skill = None

        # Load all skills data
        self.all_skills_data = list(skills_list())
        self.skill_destinations.clear()
        for skill_name, _, destinations in self.all_skills_data:
            self.skill_destinations[skill_name] = destinations

        # Show search bar
        search_bar = self.query_one("#search-bar", Input)
        search_bar.value = ""
        search_bar.styles.display = "block"
        tabs = self.query_one("#skills-tabs", Tabs)
        tabs.styles.display = "block"
        tabs.active = self.active_tab
        self.query_one("#skill-info", Static).styles.display = "none"

        # Populate table
        self._filter_skills("", focus_table=True)

    def _filter_skills(self, search_query: str, focus_table: bool = False) -> None:
        table = self.query_one("#main-table", DataTable)
        table.clear(columns=True)
        table.add_columns("skill", "installed", "path")
        table.cursor_type = "row"
        self.row_key_to_skill: dict[str, str] = {}
        self.row_key_to_path: dict[str, str] = {}
        self.row_key_to_destinations: dict[str, list[str]] = {}

        search_lower = search_query.lower()
        for skill_name, skill_path, destinations in self.all_skills_data:
            if search_lower and not (
                search_lower in skill_name.lower()
                or search_lower in skill_path.lower()
            ):
                continue
            if self.active_tab == "installed" and not destinations:
                continue
            installed = ",".join(destinations) if destinations else "-"
            row_key = f"{skill_name}::{skill_path}"
            self.row_key_to_skill[row_key] = skill_name
            self.row_key_to_path[row_key] = skill_path
            self.row_key_to_destinations[row_key] = destinations
            table.add_row(skill_name, installed, skill_path, key=row_key)

        if table.row_count > 0:
            table.move_cursor(row=0)

        if focus_table:
            table.focus()

    def _show_commands_view(self, refresh: bool = False) -> None:
        if not refresh:
            self.view_stack.append("commands")
        # Hide search bar
        search_bar = self.query_one("#search-bar", Input)
        search_bar.styles.display = "none"
        tabs = self.query_one("#skills-tabs", Tabs)
        tabs.styles.display = "none"
        skill_info = self.query_one("#skill-info", Static)
        if self.current_skill:
            description = self._read_skill_description(self.current_skill_path)
            path_value = self.current_skill_path or "-"
            desc_value = description or "-"
            skill_info.update(
                f"Skill: {self.current_skill}\n\nDescription: {desc_value}\n\nPath: {path_value}"
            )
            skill_info.styles.display = "block"
        else:
            skill_info.styles.display = "none"
        self.query_one("#skill-info", Static).styles.display = "none"

        table = self.query_one("#main-table", DataTable)
        table.clear(columns=True)
        table.add_columns("message")
        table.cursor_type = "none"
        table.add_row("No commands found")

    def _show_agents_view(self, refresh: bool = False) -> None:
        if not refresh:
            self.view_stack.append("agents")
        # Hide search bar
        search_bar = self.query_one("#search-bar", Input)
        search_bar.styles.display = "none"
        tabs = self.query_one("#skills-tabs", Tabs)
        tabs.styles.display = "none"
        self.query_one("#skill-info", Static).styles.display = "none"

        table = self.query_one("#main-table", DataTable)
        table.clear(columns=True)
        table.add_columns("message")
        table.cursor_type = "none"
        table.add_row("No agents found")

    def _show_action_view(self, skill_name: str, refresh: bool = False) -> None:
        if not refresh:
            self.view_stack.append("actions")
        self.current_skill = skill_name

        # Hide search bar
        search_bar = self.query_one("#search-bar", Input)
        search_bar.styles.display = "none"
        tabs = self.query_one("#skills-tabs", Tabs)
        tabs.styles.display = "none"
        self.query_one("#skill-info", Static).styles.display = "none"

        destinations = (
            self.current_skill_destinations
            if self.current_skill == skill_name and self.current_skill_destinations is not None
            else self.skill_destinations.get(skill_name, [])
        )
        has_claude_root = os.path.isdir(CLAUDE_BASE)
        has_codex_root = os.path.isdir(CODEX_BASE)
        has_claude = "claude" in destinations
        has_codex = "codex" in destinations
        is_mixed = has_claude != has_codex

        description = self._read_skill_description(self.current_skill_path)
        skill_info = self.query_one("#skill-info", Static)
        path_value = self.current_skill_path or "-"
        desc_value = description or "-"
        skill_info.update(
            f"Skill: {skill_name}\n\nDescription: {desc_value}\n\nPath: {path_value}"
        )
        skill_info.styles.display = "block"

        table = self.query_one("#main-table", DataTable)
        table.clear(columns=True)
        table.add_columns("Actions")
        table.cursor_type = "row"

        actions: list[tuple[str, str]] = []

        # Build action list based on installation status
        if has_claude_root and has_codex_root:
            if is_mixed:
                actions.append(("install_all", "Install for all"))
                actions.append(("uninstall_all", "Uninstall for all"))
            elif has_claude and has_codex:
                actions.append(("uninstall_all", "Uninstall for all"))
            else:
                actions.append(("install_all", "Install for all"))

        if has_claude_root:
            if has_claude:
                actions.append(("uninstall_claude", "Uninstall for Claude Code"))
            else:
                actions.append(("install_claude", "Install for Claude Code"))

        if has_codex_root:
            if has_codex:
                actions.append(("uninstall_codex", "Uninstall for Codex"))
            else:
                actions.append(("install_codex", "Install for Codex"))

        for action_id, action_name in actions:
            table.add_row(action_name, key=action_id)

        if table.row_count > 0:
            table.move_cursor(row=0)
        table.focus()

    def _show_confirm_view(self, action: str, dest: str, refresh: bool = False) -> None:
        if not refresh:
            self.view_stack.append("confirm")
        self.pending_action = (action, dest)

        # Hide search bar
        search_bar = self.query_one("#search-bar", Input)
        search_bar.styles.display = "none"
        tabs = self.query_one("#skills-tabs", Tabs)
        tabs.styles.display = "none"

        table = self.query_one("#main-table", DataTable)
        table.clear(columns=True)
        label = "all" if dest == "all" else dest
        table.add_columns(f"Confirm {action} for {label}?")
        table.cursor_type = "row"
        table.add_row("Yes", key="confirm")
        table.add_row("No", key="cancel")
        table.move_cursor(row=0)
        table.focus()

    def action_ignore(self) -> None:
        pass

    def action_focus_search(self) -> None:
        if self.view_stack[-1] == "skills":
            search_bar = self.query_one("#search-bar", Input)
            search_bar.focus()

    def action_cycle_tab(self) -> None:
        if self.view_stack[-1] != "skills":
            return
        self._set_active_tab("installed" if self.active_tab == "skills" else "skills")

    def action_cycle_tab_reverse(self) -> None:
        self.action_cycle_tab()

    def action_select_row(self) -> None:
        # Check if search bar currently has focus
        focused = self.focused
        if focused and focused.id == "search-bar":
            # Move focus to table instead of selecting
            table = self.query_one("#main-table", DataTable)
            table.focus()
            return
        table = self.query_one("#main-table", DataTable)
        table.action_select_cursor()

    def action_select_row_alt(self) -> None:
        self.action_select_row()

    def action_go_back_alt(self) -> None:
        self.action_go_back()

    def action_go_back(self) -> None:
        # If search bar has focus in skills view, clear search and return to table
        focused = self.focused
        if focused and focused.id == "search-bar" and self.view_stack[-1] == "skills":
            search_bar = self.query_one("#search-bar", Input)
            search_bar.value = ""
            self._filter_skills("", focus_table=True)
            return

        if len(self.view_stack) == 1:
            return
        self.view_stack.pop()
        if self.view_stack[-1] == "skills":
            self._show_skills_view(refresh=True)
        elif self.view_stack[-1] == "actions" and self.current_skill:
            self._show_action_view(self.current_skill, refresh=True)
        elif self.view_stack[-1] == "commands":
            self._show_commands_view(refresh=True)
        elif self.view_stack[-1] == "agents":
            self._show_agents_view(refresh=True)
        elif self.view_stack[-1] == "confirm" and self.pending_action:
            action, dest = self.pending_action
            self._show_confirm_view(action, dest, refresh=True)

    def _handle_skill_selection(self, row_key) -> None:
        skill_key = row_key.value
        skill_name = self.row_key_to_skill.get(skill_key, skill_key)
        self.current_skill_path = self.row_key_to_path.get(skill_key)
        self.current_skill_destinations = self.row_key_to_destinations.get(skill_key)
        self._show_action_view(skill_name)

    def _handle_action_selection(self, row_key) -> None:
        action_id = row_key.value

        # Parse action
        if action_id.startswith("install_"):
            action = "install"
            dest = action_id.replace("install_", "")
            # Install doesn't need confirmation
            self._execute_action(action, dest)
        elif action_id.startswith("uninstall_"):
            action = "uninstall"
            dest = action_id.replace("uninstall_", "")
            # Uninstall needs confirmation
            self._show_confirm_view(action, dest)

    def _handle_confirm_selection(self, row_key) -> None:
        key_value = row_key.value
        if key_value == "confirm" and self.pending_action:
            action, dest = self.pending_action
            self.pending_action = None
            self._execute_action(action, dest)
        else:
            # Cancel
            self.action_go_back()

    def _execute_action(self, action: str, dest: str) -> None:
        if not self.current_skill:
            return
        try:
            if action == "install":
                source_root = (
                    os.path.dirname(self.current_skill_path)
                    if self.current_skill_path
                    else None
                )
                skills_install(
                    self.current_skill,
                    "auto" if dest == "all" else dest,
                    source_root=source_root,
                )
            elif action == "uninstall":
                source_root = (
                    os.path.dirname(self.current_skill_path)
                    if self.current_skill_path
                    else None
                )
                skills_uninstall(
                    self.current_skill,
                    "auto" if dest == "all" else dest,
                    source_root=source_root,
                )
            else:
                return
            # Refresh skill data
            matches = skills_list(pattern=f"^{re.escape(self.current_skill)}$")
            if matches:
                chosen = None
                if self.current_skill_path:
                    for _, skill_path, destinations in matches:
                        if skill_path == self.current_skill_path:
                            chosen = destinations
                            break
                if chosen is None:
                    _, _, chosen = matches[0]
                self.skill_destinations[self.current_skill] = chosen
                self.current_skill_destinations = chosen
            # Go back to skills view
            while len(self.view_stack) > 1:
                self.view_stack.pop()
            self._show_skills_view(refresh=True)
        except (RuntimeError, ValueError) as exc:
            # Show error in the table
            table = self.query_one("#main-table", DataTable)
            table.clear(columns=True)
            table.add_columns("Error Message", "Details")
            table.cursor_type = "none"
            error_msg = str(exc)
            table.add_row("Action failed", error_msg)
            # Add a helpful message row
            table.add_row("Press ESC", "to go back")
            table.focus()

    def on_data_table_row_selected(self, event: DataTable.RowSelected) -> None:
        if self.view_stack[-1] == "skills":
            self._handle_skill_selection(event.row_key)
        elif self.view_stack[-1] == "actions":
            self._handle_action_selection(event.row_key)
        elif self.view_stack[-1] == "confirm":
            self._handle_confirm_selection(event.row_key)

    def on_input_changed(self, event: Input.Changed) -> None:
        if event.input.id == "search-bar" and self.view_stack[-1] == "skills":
            self._filter_skills(event.value)

    def on_tabs_tab_activated(self, event: Tabs.TabActivated) -> None:
        if self.view_stack[-1] != "skills":
            return
        if event.tab.id:
            self._set_active_tab(event.tab.id)

    def on_input_submitted(self, event: Input.Submitted) -> None:
        # Enter key handling is done in action_select_row
        pass

    def _read_skill_description(self, skill_path: str | None) -> str:
        if not skill_path:
            return ""
        skill_file = os.path.join(skill_path, "SKILL.md")
        try:
            with open(skill_file, "r", encoding="utf-8") as handle:
                contents = handle.read()
        except OSError:
            return ""

        try:
            post = frontmatter.loads(contents)
        except Exception:
            return ""
        description = post.get("description")
        if description is None:
            return ""
        return str(description).strip()
        return ""

    def _set_active_tab(self, tab_id: str) -> None:
        if tab_id not in {"skills", "installed"}:
            return
        self.active_tab = tab_id
        tabs = self.query_one("#skills-tabs", Tabs)
        tabs.active = tab_id
        search_bar = self.query_one("#search-bar", Input)
        self._filter_skills(search_bar.value)


@click.group(invoke_without_command=True)
@click.pass_context
def cli(ctx: click.Context) -> None:
    if ctx.invoked_subcommand is None:
        app = SkgetApp()
        app.run()


cli.add_command(skills)


if __name__ == "__main__":
    cli()
